# Copyright (C) 2023 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
from typing import Any

from .dependencyconfig import DependencyConfig
from .requirement import Requirement
from .provider import inject
from .provider import provide


__all__: list[str] = [
    'configure',
    'inject',
    'Require'
]


def Require(name: str) -> Any:
    return Requirement(name)


def configure(fn: str):
    config = DependencyConfig.open(fn)
    for dep in config.dependencies:
        provide(dep)